local adm = {
  _VERSION     = "adm v0.1.0",
  _DESCRIPTION = "AdMob wrapper for admob ports (requires scrale and beholder)",
  _URL         = "",
  _LICENSE     = [[
    MIT LICENSE
    Copyright (c) 2018 Martin Braun
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the
    "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish,
    distribute, sublicense, and/or sell copies of the Software, and to
    permit persons to whom the Software is furnished to do so, subject to
    the following conditions:
    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  ]]
}

-- local BANNER_HEIGHT = 50

local s, b, initGameHeight, initGameOffsetY

function adm.showBanner() if not love.ads then return nil end
	if not adm.bannerIsVisible then
		-- if adm.bannerPos == "top" and initGameOffsetY < BANNER_HEIGHT then 
		-- 	s.oY = BANNER_HEIGHT
		-- end
		-- local cW, cH = s:getCanvasSize()
		-- s.canvas = love.graphics.newCanvas(cW, initGameHeight - BANNER_HEIGHT / s.slY) -- reserve space for banner
		love.ads.showBanner()
		adm.bannerIsVisible = true
	end
end

function adm.hideBanner() if not love.ads then return nilg end
	if adm.bannerIsVisible then
		-- s.oY = initGameOffsetY
		-- local cW, cH = s:getCanvasSize()
		-- s.canvas = love.graphics.newCanvas(cW, initGameHeight) -- release reserved space
		love.ads.hideBanner()
		adm.bannerIsVisible = false
		-- love.ads.createBanner(adm.bannerId, adm.bannerPos)
	end
end

function adm.tryShowInterstitial(resultWhenNoSupport, onSuccess, onCloseAfterSuccess, onFail)
	love.interstitialClosed = function() 
		if love.ads then love.ads.requestInterstitial(adm.interstitialId) end
		if onCloseAfterSuccess then onCloseAfterSuccess() end
	end
	love.interstitialFailedToLoad = onFail
	local result, noSupport
	if love.ads then 
		result = love.ads.isInterstitialLoaded()
		if result then
			love.ads.showInterstitial()
		end
	else
		result = resultWhenNoSupport
		noSupport = true
	end
	if result and onSuccess then
		onSuccess()
	elseif not result and onFail then
		onFail()
	end
	if noSupport and result and onCloseAfterSuccess then
		onCloseAfterSuccess()
	end
	return result
end

function adm.requestRewardedAd(id)
	if love.ads then
		love.ads.requestRewardedAd(id)
	end
end

function adm.tryShowRewardedAd(resultWhenNoSupport, onSuccess, onCloseAfterSuccess, onFail)
	love.rewardedAdDidStop = onCloseAfterSuccess
	love.rewardedAdFailedToLoad = onFail
	local result, noSupport
	if love.ads then 
		result = love.ads.isRewardedAdLoaded()
		if result then
			love.ads.showRewardedAd()
		end
	else
		result = resultWhenNoSupport
		noSupport = true
	end
	if result and onSuccess then
		onSuccess()
	elseif not result and onFail then
		onFail()
	end
	if noSupport and result and onCloseAfterSuccess then
		onCloseAfterSuccess()
	end
	return result
end

function adm.init(bannerId, bannerPos, interstitialId, hideBannerOnStartup, scraleLib, beholderLib) if not love.ads then return nil end -- mobile support only
	initGameHeight = scrale.gH
	initGameOffsetY = scrale.oY
	adm.interstitialId = interstitialId
	adm.bannerId = bannerId
	adm.bannerPos = bannerPos or "bottom"
	s = scrale or scraleLib
	b = beholder or beholderLib
	if adm.bannerId then
		love.ads.createBanner(adm.bannerId, adm.bannerPos)
		if not hideBannerOnStartup then 
			adm.showBanner() 
			adm.bannerIsVisible = true
		end
	end
	if adm.interstitialId then
		love.ads.requestInterstitial(adm.interstitialId)
	end
	if b then
		love.rewardUserWithReward = function(rewardType, rewardQuantity)
			b.trigger("ADS.REWARD", rewardType, rewardQuantity)
		end
	end
end

return adm