local slog = {
  _VERSION     = "slog v0.1.0",
  _DESCRIPTION = "Simple Logger for LÖVE",
  _URL         = "",
  _LICENSE     = [[
    MIT LICENSE
    Copyright (c) 2018 Martin Braun
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the
    "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish,
    distribute, sublicense, and/or sell copies of the Software, and to
    permit persons to whom the Software is furnished to do so, subject to
    the following conditions:
    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  ]]
}
-- TODO: Add possibility to remove old files, automatically

local maxLLc, maxLLf = 6, 6
local fdir, fname, fdateformat = "logs", "{DATE}.log", "%Y_%m_%d"
local mformat = "%Y-%m-%d %H:%M:%S [%%L] %%B%%M"
local file

local llnames = { "SYSTEM", "FATAL", "ERROR", "WARN", "INFO", "DEBUG" }
local blockstrs = { "", "███!!!███", "██!!██", "█!█", "", "" }
slog.LEVEL = {}

function slog.init(maxLogLevelConsole, maxLogLevelFile, logfileDirectory, logfileName, logfileDateFormat, logmessageFormat)
	maxLLc, maxLLf = maxLogLevelConsole or maxLLc, maxLogLevelFile or maxLLf
	fdir, fname, fdateformat = logfileDirectory or fdir, logfileName or fname, logfileDateFormat or fdateformat
	mformat = logmessageFormat or mformat

	for i = 1, #llnames do
		slog.LEVEL[llnames[i]] = i
	end

	if maxLLf > 0 then
		local t = os.date(fdateformat)
		local rfname = string.gsub(fname, "{DATE}", t)
		if string.len(fdir) > 0 then
			love.filesystem.createDirectory(fdir)
			fdir = fdir .. "/"
		end
		file = love.filesystem.newFile(fdir .. rfname)
		file:open("a")
	end
end

function slog.log(level, message, ...)
	if level <= maxLLc or level <= maxLLf then
		for i = 1, select("#", ...) do
			message = string.gsub(message, "%%" .. i, tostring(select(i, ...)))
		end
		message = string.gsub(os.date(mformat), "%%M", message)
		message = string.gsub(message, "%%B", blockstrs[level])
		message = string.gsub(message, "%%L", llnames[level])
		if level == 2 or level == 3 then
			message = message .. "\n        at " .. debug.traceback()
		end
		if level <= maxLLc then
			print(message)
		end
		if file and level <= maxLLf then
			file:write(message .. "\r\n")
		end
		if level == 2 then
			love.event.quit(91)
		end
	end
end

return slog